/*
madman - a music manager
Copyright (C) 2003  Andreas Kloeckner <ak@ixion.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/




#ifndef MAINWIN
#define MAINWIN




#include <memory>
#include <qtimer.h>
#include <qlistview.h>
#include <qapplication.h>

#include "designer/mainwin_ui.h"
#include "database/database.h"
#include "httpd/httpd.h"
#include "httpd/madmanweb_external.h"
#include "utility/prefs.h"
#include "ui/prefs_dialog.h"
#include "ui/song_set_view.h"
#include "ui/overview.h"




class tSongSetNodeListViewItem;
class tProgress;




class tMainWindow : public tMainWindowBase, public tProgramInterface
{
    Q_OBJECT;

    auto_ptr<tDatabase> Database;

    tSongSet AllSongsSet;
    tAllSongsViewManager AllSongsView;
    tEditableSongSetViewManager SongSetEditor;

    auto_ptr<tCriterion> SearchCriterion;
    QSettings		Settings;
    tPreferences	Preferences;

    tFilename CurrentFilename;
    bool FilenameValid;

    tFilename CurrentSongFilename;
    double AccumulatedPlayTime;
    time_t PlayStartTime;
    bool WasPlaying;
    QTimer UpdateTimer;

    tProgress *CurrentSearchProgress;
    tHttpDaemon *HttpDaemon;

    tOverviewManager OverviewManager;

  public:
    tMainWindow( const QString &filename_to_open );
    virtual ~tMainWindow();

    void setDatabase( tDatabase *db );

    void closeEvent( QCloseEvent *event );

    void setupListView( tSongSetViewManager &mgr, bool save_before );
    void updateWindowCaption();
    void updateAll();

    tDatabase &database()
    {
      return *Database;
    }
    tPreferences &preferences()
    {
      return Preferences;
    }
    QSettings &settings()
    {
      return Settings;
    }

    void quitApplication()
    {
      qApp->closeAllWindows();
    }

  public slots:
    void rescan();
    void rescan( tDatabase *db );
    void rereadTags();
  
    void fileNew();
    void fileOpen();
    void saveDatabase( const QString &name );
    void fileSave();
    bool fileSaveWithResult();
    void fileSaveAs();
    bool fileSaveAsWithResult();
    void filePreferences();
    
    void helpAbout();
    void help();

    void addSongSet( tSongSetNode *node );
    void addSongSet();
    void bookmarkCurrentSearch();
    void importSongSet();
    void duplicateSongSet();
    void removeSongSet();
    void editMultilineSongSetCriterion();
    void slotDropSongSetNode( const QString &node, tSongSetNode *onto );
    void renameSongSet( QListViewItem *item, int col, const QString &text );
    void songSetSelectionChanged();
    void songSetCriterionChanged();

    void add( tSongList const &songlist );

    void searchChanged();
    void searchChanged( const QString &text )
    {
      searchChanged();
    }

    void buildOverviewTree();
    void overviewSelectionChanged( QListViewItem *item );

    void highlightCurrentSong();

    void play();
    void pause();
    void stop();
    void skipForward();
    void skipBack();
    void skipTo( int value );
    void skipToWrapper();

    void updatePlayerStatus();
    void songOrStateChanged();
    void playerPossiblyChanged();

  private:
    void updateSongSetTree( bool keep_selection, bool keep_scroll );
    tSongSetNode *nodeFromItem( QListViewItem *item );
    tSongSetNode *currentNode();
    tSongSet *currentSongSet();
    void insertAtPosition( QListViewItem *parent, QListViewItem *item, tIndex pos );

    QListViewItem *createSongSetNodeItem( tSongSetNode *node, QListViewItem *parent,
	tSongSetNode *selected_node, tSongSetNodeListViewItem *&selected_item );

    void loadSplitterAppearance( QSplitter *splitter, QSettings &settings, QString const &where );
    void saveSplitterAppearance( QSplitter *splitter, QSettings &settings, QString const &where );
};




#endif
